#### REDÉCOUPAGE AUTOMATIQUE DES CIRCONSCRIPTIONS LÉGISLATIVES FRANÇAISES ####
####           @ Adrien Fabre, adrien.fabre@psemail.eu 2017               ####
####             Librement réutilisable avec attribution                  ####

linux <- TRUE
if (linux) { options(download.file.method = "wget") # For Ubuntu 14.04
} else options(download.file.method = "auto")

package <- function(p) { 
  if (!is.element(p, installed.packages()[,1])) {
    install.packages(p); 
  }
  library(p, character.only = TRUE)
} # loads packages with automatical install if needed

package('foreign')
package("readxl")
package("mfx")

setwd("/var/www/We give the 99 percents/Redécoupage") # /!\ Changer le chemin d'accès
load("env.RData")


##### Préparation: cantons #####
# Cantons 2015 données par OSM. Source: https://www.data.gouv.fr/fr/datasets/contours-osm-des-cantons-electoraux-departementaux-2015/
cantons_osm <- read.dbf("cantons/cantons_2015.dbf", as.is = TRUE) # cantons métropolitains
# Les données géographiques sont ensuite simplifiées (jusqu'à 10%) à l'aide de mapshaper.org pour optimiser l'utilisation de la mémoire par autoredistrict

# # Jonction de ce jeu de données avec les résultats électoraux (non utilisée in fine) :
# # Résultats 2è tour Présidentielle 2017. Source: https://www.data.gouv.fr/fr/posts/les-donnees-des-elections/
# election2017 <- read_excel("Résultats présidentielle/2017.xls")
# cantons_osm$canton <- as.numeric(cantons_osm$canton)
# cantons_osm$dep[cantons_osm$dep=="2A"] <- 120
# cantons_osm$dep[cantons_osm$dep=="2B"] <- 121
# cantons_osm$dep <- as.numeric(cantons_osm$dep)
# cantons_osm$dep[cantons_osm$dep==120] <- "2A"
# cantons_osm$dep[cantons_osm$dep==121] <- "2B"
# cantons_osm$canton <- paste(cantons_osm$dep, cantons_osm$canton)
# election2017$canton <- paste(election2017[[1]], election2017$X__2)
# election2017$Macron <- as.numeric(election2017$X__22)
# election2017$LePen <- as.numeric(election2017$X__29)
# election2017 <- election2017[-c(1:3),-c(1:32)]
# cantons_osm <- merge(cantons_osm, election2017, sort=FALSE)
# 
# # Nettoyage, rajout d'une colonne pour les nouvelles circos et exportation
# cantons_osm$circos_nouvelles <- c(1:length(cantons_osm[[1]]))
# cantons_osm <- cantons_osm[,-c(3:8)]
# cantons_osm$population <- as.numeric(cantons_osm$population)
write.dbf(cantons_osm, "cantons/cantons simple.dbf")


##### Préparation: Français de l'étranger #####
# Le nombre d'inscrits sur les données électorales de 2017 ont été utilisées pour déterminer la population française dans chaque pays
# Source: https://www.data.gouv.fr/s/resources/elections-presidentielles-2017-2nd-tour-vote-des-francais-etablis-hors-de-france/20170509-121225/Presidentielle2017-FAE-2nd-tour-pays.csv
# Source des données géographiques "étranger": http://thematicmapping.org/downloads/TM_WORLD_BORDERS_SIMPL-0.3.zip
# Afin de combiner les données démographiques/électorales avec les données géographiques, j'ai inséré le Kosovo dans l'Albanie et Jérusalem dans Israël (directement dans le .csv)
#  et j'ai recoder les noms de pays par le code FIPS présent dans les données géographiques
etranger2017 <- read.csv("Résultats présidentielle/2017 étranger.csv", stringsAsFactors=FALSE, sep=";", fileEncoding="latin1") # données démographiques/électorales
etranger2017$inscrits <- etranger2017[[2]]
etranger2017$Macron <- etranger2017[[9]]
etranger2017$exprimes <- etranger2017[[7]]
etranger2017 <- etranger2017[-c(2:12)]
monde <- read.dbf("Monde/étranger.dbf") # données géographiques originales
monde$Pays <- as.vector(monde$FIPS)
monde$Pays[is.na(monde$Pays)] <- as.vector(monde$ISO3[is.na(monde$Pays)])
monde2 <- merge(monde, etranger2017, by="Pays", all.x=TRUE, sort=FALSE)
monde2 <- merge(monde, monde2, by="Pays", all.x=TRUE, sort=FALSE)
monde2 <- monde2[,c(1,6,24:26)]
write.dbf(monde2, "Monde/2017 étranger.dbf")
# Afin d'avoir des circonscriptions faisant sens du point de vue géographique, j'ai regroupé arbitrairement certains pays
#  dans la fonction 'region_monde'. Ainsi, la recherche de la solution optimale automatique était légèrement contrainte
region_monde <- function(d) {
  df <- d
  # df$region[is.element(df$Pays,c('AG','TS','MO','LY'))] <- 'Maghreb'
  df$region[is.element(df$Pays,c('BN','UV','BY','CM','CT','CN','CF','CG','IV','DJ','GB','GV','MA','ML','MR','NG','RW','SG','CD','TO','SF','AO', 'BC', 'CV', 'ET', 'GH', 'PU', 'EK', 'KE', 'MZ', 'WA', 'NI', 'UG', 'SE', 'SU', 'ZA', 'ZI', 'MA','TZ', 'CN', 'SO', 'SL', 'ER', 'LI', 'LT', 'MI'))] <- 'Afrique'
  df$region[is.element(df$Pays,c('RP','ID','AS','NZ','PP','MY','SN','BX','FJ','NH'))] <- 'Pacifique'
  df$region[is.element(df$Pays,c('CU','DR','HA','JM','TD','CS','HO','NU', 'MX', 'GT', 'PM', 'CO', 'BH', 'RN', 'ST', 'TB', 'VE', 'BR', 'AR', 'CI', 'BL', 'PE', 'EC', 'NS', 'GY', 'UY', 'PA'))] <- 'Latino'
  df$region[is.element(df$Pays,c('UK','EI','NL', 'IC', 'GL'))] <- 'EuropeOccidentale'
  df$region[is.element(df$Pays,c('TW','JA', 'KS', 'CH', 'VN', 'KN'))] <- 'AsieOrientale'
  # df$region[is.element(df$Pays,c('MA','TZ', 'CN'))] <- 'AfriqueEst'
  df$region[is.na(df$region)] <- df$Pays[is.na(df$region)]
  df2 <- aggregate(cbind(inscrits, Macron, exprimes) ~ region, df, sum)
  df2$Pays <- df2$region
  df2$Pays[df2$region=="AsieOrientale"] <- "CH"
  df2$Pays[df2$region=="Afrique"] <- "ML"
  df2$Pays[df2$region=="EuropeOccidentale"] <- "NL"
  df2$Pays[df2$region=="Latino"] <- "PM"
  df2$Pays[df2$region=="Pacifique"] <- "MY"
  df2 <- merge(df, df2, by="Pays", all.x=TRUE, sort=FALSE)
  return(df2)
}
monde3 <- region_monde(monde2)
names(monde3) <- c('Pays', 'Nom', 'inscrits', 'Macron', 'exprimes', 'region', 'region2', 'inscrits_region', 'Macron_region', 'exprimes_region')
monde3 <- merge(monde, monde3, by="Pays", all.x=TRUE, sort=FALSE)
monde3 <- monde3[,c(1,6,14:17,19:21)]
monde3$circo <- monde3$region
write.dbf(monde3, "Monde/étranger2017_regionalise.dbf")
# réunion des régions sur QGis dans étranger2017_regionalise.shp (edit > merged selected...)


##### Préparation: populations #####
# Populations légales 2014 par communes (inclut la population de plus de 15 ans mais pas la population majeure, absente des données de l'INSEE)
# Source: https://www.insee.fr/fr/statistiques/2862200
pop <- read_excel("pop_communes_2014.xls")
pop <- pop[-c(1:5),]
names(pop) <- c("insee", "pop", "pop15")
pop$pop15 <- as.numeric(pop$pop) - as.numeric(pop$pop15)
# Je rajoute à la main les populations des arrondissements de Lyon à partir de l'excel définissant pop (elles sont dans une autre feuille du document):
pop[nrow(pop)+1,] <- c("69381", 29539,29539-4259)
pop[nrow(pop)+1,] <- c("69382", 29999, 29999-3882)
pop[nrow(pop)+1,] <- c("69383", 99819,99819-15869)
pop[nrow(pop)+1,] <- c("69384", 36101,36101-5840)
pop[nrow(pop)+1,] <- c("69385", 47302,47302-6936)
pop[nrow(pop)+1,] <- c("69386", 50419,50419-7343)
pop[nrow(pop)+1,] <- c("69387", 80993,80993-11224)
pop[nrow(pop)+1,] <- c("69388", 83619,83619-14433)
pop[nrow(pop)+1,] <- c("69389", 48824,48824-9016)

# Population par circonscription actuelle (2013) Source: https://www.insee.fr/fr/statistiques/2508230
pop_circos <- read_excel("pop_circos_2013.xls")
pop_circos <- as.numeric(pop_circos[-c(1:7),]$X__3)
# Ajout des français de l'étranger à partir de Wikipédia: https://fr.wikipedia.org/wiki/Circonscriptions_l%C3%A9gislatives_des_Fran%C3%A7ais_%C3%A9tablis_hors_de_France
# Renormalisation en termes de population totale et non d'inscrits. Source: , http://www.vie-publique.fr/focus/elections-2017-combien-electeurs-inscrits.html, https://fr.wikipedia.org/wiki/D%C3%A9mographie_de_la_France
ratio_pop_inscrits <- 67/45.7
pop_circos <- sort(c(pop_circos,ratio_pop_inscrits*c(259390,101084,173477,185502,112029,179597,151576,137560,171290,158312,152371))) 


##### Préparation: résultats électoraux #####
# election2017 <- read_excel("Résultats présidentielle/2017.xls")
# election2017$canton <- paste(election2017[[1]], election2017$X__2)
# election2017$Macron <- as.numeric(election2017$X__22)
# election2017$LePen <- as.numeric(election2017$X__29)
# election2017 <- election2017[-c(1:3),-c(1:32)]

election2012 <- read_excel("Résultats présidentielle/2012.xls")
election2012$canton <- paste(election2012[[1]], election2012[[3]])
election2012$Hollande <- election2012$Voix
election2012$Sarkozy <- election2012$Voix__1
election2012 <- election2012[,-c(1:27)]

# Résultat présidentielle	2012: circos 2012: 333/577 i.e. 57,71% / total national : 51,64	%	i.e. 279.96/577 pour Hollande
# Résultat présidentielle	2017: circos 2017: 532/577 i.e. 92,20% / total national : 61,10	%	i.e. 352.55/577 pour Macron

##### Paris #####
# Paris n'est pas divisé en cantons, mais chacun des 20 arrondissements contient 4 quartiers.
# J'ai ainsi choisi les quartiers comme unité de base pour créer les circonscriptions.
# Source des données démographiques (2012): http://opendata.apur.org/datasets/iris-demographie/geoservice (http://carto2.apur.org/apur/rest/services/OPENDATA/IRIS_DEMOGRAPHIE/MapServer/0/query?where=1%3D1&outFields=N_QU,P12_POP,P12_H0014,P12_F0014&returnGeometry=false&outSR=4326&f=json)
# Source des données géographiques: http://opendata.apur.org/datasets/quartier
quartiers_Paris <- read.csv("Paris/IRIS_DEMOGRAPHIE.csv")
quartiers_Paris <- quartiers_Paris[!is.na(quartiers_Paris$N_QU)&!is.na(quartiers_Paris$P12_POP),c("N_QU", "P12_POP", "P12_H0014", "P12_F0014")]
quartiers_Paris <- aggregate(cbind(P12_POP, P12_POP-P12_H0014-P12_F0014) ~ N_QU, quartiers_Paris, sum)
names(quartiers_Paris) <- c("quartier", "pop", "pop15")
write.dbf(quartiers_Paris, "Paris/QUARTIER.dbf")

## Initialement, j'avais calculé à part les circonscriptions parisiennes ##
# Le résultat est visible à Paris/circos_proposées.png (ou .ggs)
sum(quartiers_Paris$pop15)/160000 # 12 circonscriptions
regroupement_quartiers <- read.dbf("Paris/résultat.dbf", as.is = TRUE)
circonscriptions_Paris_proposees <- aggregate(cbind(pop, pop15) ~ quartier, regroupement_quartiers, sum)
regroupement_quartiers <- regroupement_quartiers[,-c(1)]
names(regroupement_quartiers) <- c("pop", "pop15", "circo_proposee")
quartiers_Paris$pop_rounded <- round(quartiers_Paris$pop)
regroupement_quartiers$pop_rounded <- round(regroupement_quartiers$pop)
quartiers_Paris <- merge(regroupement_quartiers, quartiers_Paris, by=c("pop_rounded"))
rm(regroupement_quartiers)
# Statistiques sur les résultats
sd(circonscriptions_Paris_proposees$pop15)/mean(circonscriptions_Paris_proposees$pop15) # 3,4%
(max(circonscriptions_Paris_proposees$pop15) - min(circonscriptions_Paris_proposees$pop15))/mean(circonscriptions_Paris_proposees$pop15) # 11%


##### Lyon Métropole ####
# Lyon Métropole n'est pas divisée en cantons. J'ai choisi comme unité de base les arrondissements lyonnais et les communes hors Lyon
# Source données géographiques: https://data.grandlyon.com/limitesadministratives/commune-du-grand-lyon-voies-et-adresses/ et https://data.grandlyon.com/limitesadministratives/arrondissement-de-lyon-voies-et-adresses/
# J'ai uni les données géographiques dans GQis, créant "Lyon/lyon reconstitué.shp"
# Ci-dessous, je combine ces données avec les données de population
communes_Lyon_dbf <- read.dbf("Lyon/lyon reconstitué.dbf", as.is = TRUE)
communes_Lyon_dbf$insee[is.na(communes_Lyon_dbf$insee)] <- communes_Lyon_dbf$inseecommu[is.na(communes_Lyon_dbf$insee)]
communes_Lyon <- communes_Lyon_dbf
communes_Lyon$nomreduit[is.na(communes_Lyon$nomreduit)] <- communes_Lyon$nomredui_2[is.na(communes_Lyon$nomreduit)]
# communes_Lyon$insee[is.na(communes_Lyon$insee)] <- communes_Lyon$inseecommu[is.na(communes_Lyon$insee)]
communes_Lyon$gid[is.na(communes_Lyon$gid)] <- communes_Lyon$gid_2[is.na(communes_Lyon$gid)]
communes_Lyon <- aggregate(communes_Lyon, list(communes_Lyon$insee), function(vec) { return(vec[1]) } )
communes_Lyon <- communes_Lyon[,c(3,4,6)]
names(communes_Lyon) <- c("commune", "insee", "gid")
communes_Lyon <- merge(communes_Lyon, pop)
communes_Lyon$circou_nouvelle <- communes_Lyon$insee
write.dbf(communes_Lyon, "Lyon/lyon.dbf")

## Initialement, j'avais calculé à part les circonscriptions parisiennes ##
sum(as.numeric(communes_Lyon$pop))/200000 # 7 circonscriptions à déterminer (6.77)
circos_Lyon_nouveau <- read.dbf("Lyon/résultat.dbf", as.is = TRUE)
circos_Lyon_nouveau <- aggregate(cbind(pop, pop15) ~ circou_nou, circos_Lyon_nouveau, sum)
# Statistiques sur les résultats
sd(circos_Lyon_nouveau$pop15)/mean(circos_Lyon_nouveau$pop15) # 3.0%
(max(circos_Lyon_nouveau$pop15) - min(circos_Lyon_nouveau$pop15))/mean(circos_Lyon_nouveau$pop15) # 8.8%


##### Inclusion de Paris et Lyon dans l'Hexagone #####
# Regroupement des données Paris, Lyon Métropole et l'Hexagone
paris <- read.dbf("Paris/QUARTIER.dbf", as.is=TRUE)
lyon <- read.dbf("Lyon/lyon.dbf", as.is=TRUE)
france <- read.dbf("cantons/cantons simple.dbf", as.is=TRUE)
paris$atom <- paris$quartier
lyon$atom <- lyon$insee
france$atom <- france$canton
france$pop <- france$population
paris <- paris[c('pop', 'atom')]
lyon <- lyon[c('pop', 'atom')]
france <- france[c('pop', 'atom')]
write.dbf(paris, "métropole/paris.dbf")
write.dbf(lyon, "métropole/lyon.dbf")
write.dbf(france, "métropole/france.dbf")
# Union géographique de lyon et france dans metropole.shp avec QGis, dont nettoyage des doublons lyonnais dans les attributs (Vector > Geoprocessing.. > Union)
metropole <- read.dbf("métropole/metropole.dbf", as.is=TRUE)
metropole$atom[is.na(metropole$atom)] <- metropole$atom_2[is.na(metropole$atom)]
metropole$pop[is.na(metropole$pop)] <- metropole$pop_2[is.na(metropole$pop)]
metropole <- metropole[c('pop', 'atom')]
write.dbf(metropole, "métropole/metropole.dbf")
# Union de paris et metropole dans tout.shp après chgt du CRS de paris (save as...), puis nettoyage des doublons dans Qgis (s'aider de duplicated(tout$atom))
tout <- read.dbf("métropole/tout2.dbf", as.is=TRUE)
tout$atom[is.na(tout$atom)] <- tout$atom_2[is.na(tout$atom)]
tout$pop[is.na(tout$pop)] <- tout$pop_2[is.na(tout$pop)]
tout <- tout[c('pop', 'atom')]
tout$circo <- tout$atom
tout$pop <- round(as.numeric(tout$pop))
write.dbf(tout, "métropole/tout.dbf")


##### Répartition des députés entre métropole et le reste #####
# Il y a 313 circonscriptions à répartir d'après ce rapport p.28 : http://www2.assemblee-nationale.fr/static/reforme-an/Rapport-1-GT.pdf
# Je fais le choix de séparer l'Hexagone, la Corse, chaque DROM, la Polynésie et les français établis à l'étranger dans des ensembles de circonscriptions distincts
# Je n'ai pas inclus la Nouvelle-Calédonie, dont le statut va sûrement changé en 2018.
# Dans un souci de cohérence avec le découpage automatique, c'est la population totale qui est considérée.
# Dans l'absolu, il faudrait répartir les nombre de sièges à partir de la population majeure.
# Les données non contenues dans pop ont été obtenues sur Wikipedia: https://fr.wikipedia.org/wiki/Collectivit%C3%A9_d%27outre-mer et https://fr.wikipedia.org/wiki/Fran%C3%A7ais_%C3%A9tablis_hors_de_France
pop_tot <- sum(as.numeric(pop$pop))+240000+331834+1710945 # Population totale 2014, où j'ai rajouté Mayotte, les COM et les français de l'étranger (2015) à la main
pop_par_circo <- pop_tot/313  # 219478 habitants par circo
sum(as.numeric(pop$pop[as.numeric(pop$insee)<96000|is.na(as.numeric(pop$insee))]))/pop_par_circo # 294 circos pour métropole + Corse (294.04)
1710945/pop_par_circo # 8 circos pour français de l'étranger (7.80)
331834/pop_par_circo # 2 circos pour les COM (Polynésie, etc.) (1.51), mais je propose de ne pas mettre dans une même circo des îles de l'Atlantique et du Pacifique :
(268707+12197)/pop_par_circo # 1 circo pour la Polynésie + Wallis-et-Futuna (1.28)
sum(as.numeric(pop$pop[is.na(as.numeric(pop$insee))]))/pop_par_circo # 1 circo pour la Corse (1.48)
(240000+sum(as.numeric(pop$pop[as.numeric(pop$insee)>96000&!is.na(as.numeric(pop$insee))])))/pop_par_circo # 10 circos pour les DROM (9.69)
sum(as.numeric(pop$pop[as.numeric(pop$insee)<96000&!is.na(as.numeric(pop$insee))]))/pop_par_circo # 293 circos pour l'Hexagone (métropole hors Corse) (292.56)
# Je fais le choix de séparer les COM et de créer une circonscription Polynésie+Wallis-et-Futuna (dans le Pacifique) 
# et de ratacher les îles de l'Atlantique à des circonscriptions de la Martinique et de la Guadeloupe
# On aboutit bien à un total de 313 circos (ou 314 si on laisse 2 circos à la Corse), ce qui laisse 90 (ou 89) députés à la proportionnelles 
# Le découpage est effectué automatiquement pour l'Hexagone, pour les français de l'étranger et pour la Réunion
# Pour la Corse, la Guyane, la Martinique, la Guadeloupe, Mayotte, et les autres îles, j'ai effectué le découpage moi-même, car il n'y avait qu'une issue cohérente
# La répartition dans les DROM est la suivante:
sum(as.numeric(pop$pop[grepl('971', pop$insee)]))/pop_par_circo # 2 Guadeloupe (1.82)
sum(as.numeric(pop$pop[grepl('972', pop$insee)]))/pop_par_circo # 2 Martinique (1.75)
(sum(as.numeric(pop$pop[grepl('971', pop$insee)]))+sum(as.numeric(pop$pop[grepl('972', pop$insee)]))+331834-268707)/pop_par_circo #  Guadeloupe, Martinique et COM autres que Polynésie*: 3.86 *(Walis-et-Futuna 12 197, Saint-Martin 35 594, Saint-Barthélémy 9 279 et Saint-Pierre-et-Miquelon 6 057)
sum(as.numeric(pop$pop[grepl('973', pop$insee)]))/pop_par_circo # 1 Guyane (1.15)
sum(as.numeric(pop$pop[grepl('974', pop$insee)]))/pop_par_circo # 4 Réunion (3.84)


##### Autoredistrict.org #####
# Exécuter autoredistricting à partir de "tout.shp", "Monde/étranger2017_regionalise.shp" et "cantons/Réunion.shp"
# Pour ce faire: en ligne de commande, se placer dans le bon dossier, et pour lancer autoredistricting:
# java -jar autoredistrict.jar -Xmx4096M -Xms1024M
# J'ai utilisé les paramètres suivants pour l'Hexagone : num. of districts: 293, pop. column: pop, district column: circo
# anneal rate: 3%, mutation: 95%, elitism: 40%, elites mutated: 100%
# geometric: 100%, contiguous: 80% (non coché), compact: 5%, equal pop: 100% (coché)
# Ces paramètres ont été utilisés à la fin des 50.000 itérations (ce qui a pris un jour de calculs), avant, j'avais pris
# les mêmes paramètres sauf elitism: 5%, elites mutated: 0%, et equal pop non coché.


##### Résultats: préparation #####
cantons_resultat <- read.dbf("Résultats/résultat.dbf", as.is = TRUE) # À chaque canton est associé une circo
circos_nouvelles <- aggregate(pop ~ circo, cantons_resultat, sum) # Les populations de chaque circo
# Ajout des régions hors Hexagone
circos_monde <- read.dbf("Monde/resultat moins naif.dbf", as.is = TRUE) # À chaque pays/région est associé une circo des français de l'étranger
circos_monde$circo <- paste('etr', circos_monde$c.AC..Anti)
circos_monde <- aggregate(cbind(inscrits_r, exprimes_r) ~ circo, circos_monde, sum) # Les populations de chaque circo
# Renormalisation des populations des français de l'étranger, seules exprimées en nombre d'inscrits plutôt qu'en population totale
circos_monde$pop <- circos_monde$inscrits_r*mean(circos_nouvelles$pop)/mean(circos_monde$inscrits_r)
circos_nouvelles <- merge(circos_nouvelles, circos_monde[,c(1,4)], all=T)
# Choix d'une circonscription corse, on aurait pu en mettre deux
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Corse", pop = sum(as.numeric(pop$pop[grepl('2A',pop$insee)|grepl('2B',pop$insee)])))
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Polynésie&Walis", pop = 268707+12197)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Mayotte", pop = 240000)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Guyane", pop = sum(as.numeric(pop$pop[grepl('973', pop$insee)])))
# La Guadeloupe et la Martinique sont divisées en deux circonscriptions ayant autant d'arrondissements
# On attribue Saint-Barthélémy et Saint-Pierre-et-Miquelon à Basse-Terre (Guadeloupe) et Saint-Martin à Fort-de-France (Martinique)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "St-Barthélémy&St-Pierre-et-Miquelon&Guadeloupe_Basse-Terre", pop = 9279+6057+190255)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Guadeloupe_Pointe-a-Pitre", pop = 209931)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "Martinique_Marin&Trinite", pop = 119693+80415)
circos_nouvelles[length(circos_nouvelles$pop)+1,] <- c(circo = "St-Martin&Martinique_Fort-France&St-Pierre", pop = 35594+160498+23305)
reunion_resultat <- read.dbf("cantons/Réunion.dbf", as.is = TRUE)
circos_reunion <- aggregate(population ~ ref, reunion_resultat, sum)
circos_reunion$circo <- paste('reu', circos_reunion$ref)
circos_reunion <- circos_reunion[,c(2,3)]
names(circos_reunion) <- c('pop', 'circo')
circos_nouvelles <- merge(circos_nouvelles, circos_reunion, all=T)
circos_nouvelles$pop <- as.numeric(circos_nouvelles$pop)


##### Résultats: statistiques #####
# Exemples de circos non contigues : 33 19 (!) / 18 1 / 44 4 /40 5 / 42 4 / 6 11 / 77 12 / 59 12 / 7 13 / 15 11 / 4 9 / 17 4 / 2 3 / 12 3 / 34 10 / 9 11 

# Différence de population entre les extrêmes (en proportion de la moyenne), actuel:
(max(pop_circos)-min(pop_circos))/mean(pop_circos) #  Max-min: 313%
(pop_circos[576]-pop_circos[4])/mean(pop_circos) #  Max-min: 174% Hexagone seulement
# et dans le redécoupage proposé:
(max(circos_nouvelles$pop) - min(circos_nouvelles$pop))/mean(circos_nouvelles$pop) #  Max-min: 65%
(max(circos_nouvelles$pop[circos_nouvelles$pop<270000]) - min(circos_nouvelles$pop))/mean(circos_nouvelles$pop[circos_nouvelles$pop<270000])
#  Max-min: 32% en enlevant la Corse et la Polynésie&Wallis

# Average absolute difference (i.e. écart moyen entre les populations de deux circonscriptions aléatoires: E[|X-Y|]):
aad <- aad_nou <- 0
for (i in 1:577) {  for (j in 1:577) {    aad <- aad + abs(pop_circos[i] - pop_circos[j])  } }
for (i in 1:313) {  for (j in 1:313) {    aad_nou <- aad_nou + abs(circos_nouvelles$pop[i] - circos_nouvelles$pop[j])  } }
aad <- aad/577^2/mean(pop_circos) # actuel: 17.7% de la moyenne
aad_nou <- aad_nou/313^2/mean(circos_nouvelles$pop) # proposé: 6.1% de la moyenne

# Écart-type actuel (en proportion de la moyenne)
sd(pop_circos)/mean(pop_circos) # sd: 20.5%
# et dans le redécoupage proposé:
sd(circos_nouvelles$pop)/mean(circos_nouvelles$pop)
# sd: 6.2% de 216k, la taille moyenne
sd(circos_nouvelles$pop[circos_nouvelles$pop<270000])/mean(circos_nouvelles$pop[circos_nouvelles$pop<270000])
# sd: 5.2% en enlevant la Corse et la Polynésie&Wallis

# Nombre de circonscriptions dont la population s'écarte de plus de 20% de la moyenne, par le dessus puis par le dessous
nb_inf <- length(which(pop_circos<0.8*mean(pop_circos))) # 47, soit 8.1%
nb_sup <- length(which(pop_circos>1.2*mean(pop_circos))) # 24, soit (24+47)/577=12.3%
# et dans le redécoupage proposé:
nb_inf_nou <- length(which(circos_nouvelles$pop<0.8*mean(circos_nouvelles$pop))) # 0
nb_sup_nou <- length(which(circos_nouvelles$pop>1.2*mean(circos_nouvelles$pop))) # 2


##### Graphique #####
plot(seq(0,1, by=1/576), quantile(pop_circos/mean(pop_circos), probs=seq(0,1, by=1/576)), axes=FALSE, ylab="Population (par rapport à la moyenne)", xlab="Circonscriptions, des moins peuplées au plus peuplées",  type='s', lwd=2) + grid()
lines(seq(0,(nb_inf-1)/576, by=1/576), pop_circos[1:nb_inf]/mean(pop_circos), col="red", type="s", lwd=2)
lines(seq((577-nb_sup)/576,1, by=1/576), pop_circos[(578-nb_sup):577]/mean(pop_circos), col="red", type="s", lwd=2)
legend("topleft", title="Population des circonscriptions en proportion de la moyenne", text.col=c("black", "green", "red"), lwd=2, lty=1, col=c("black", "green", "red"), legend=c("Découpage actuel", "Découpage proposé", "Taille s'écartant d'au moins 20% de la moyenne"))
lines(seq(0,1,by=1/312), sort(circos_nouvelles$pop)/mean(circos_nouvelles$pop), col="green", lwd=2, type='s')
lines(seq(311/312,1,by=1/312), sort(circos_nouvelles$pop[which(circos_nouvelles$pop>1.2*mean(circos_nouvelles$pop))])/mean(circos_nouvelles$pop), col="red", lwd=2, type='s')
abline(h=seq(0.8,1.2,by=0.4), col="grey",lty=2) + abline(h=1, col="grey",lty=3)
axis(2, at=c(0,0.4,0.8, 1.2,1.6,2,2.4,2.8)) + axis(1) + box()


##### Corrélations taille/résultats électoraux #####
# Données électorales de 2012 par canton. Source: https://www.data.gouv.fr/s/resources/election-presidentielle-des-23-avril-et-7-mai-2017-resultats-definitifs-du-2nd-tour/20170511-092258/Presidentielle_2017_Resultats_Tour_2_c.xls
res_2012 <- read.csv2("Résultats présidentielle/2012_minimal.csv")
res_2012$Hollande <- as.numeric(as.vector(res_2012$Hollande))
sum(res_2012$Hollande>50)/577 # Hollande aurait obtenu 57.71% de sièges théoriques contre 51.64% de score national
plot(res_2012$Inscrits, res_2012$Hollande) # On voit que Saint-Pierre-et-Miquelon profite nettement à Hollande. Pour le reste, l'effet est moins marqué, mais quand même présent (cf. ci-dessous)
cor(res_2012$Hollande, res_2012$Inscrits) # -0.19 de corrélation entre le nombre d'inscrits et le score de Hollande
summary(lm(res_2012$Hollande ~ res_2012$Inscrits)) # -1.3e-4*** Score de Hollande régressé sur nombre d'inscrits
# !!! 12660 inscrits en moins est associé avec 1% de plus pour Hollande
summary(lm(I(res_2012$Hollande>50) ~ res_2012$Inscrits)) # -3.7e-6* Linear probability model:
# i.e. 37k inscrits en plus augmentent la proba de victoire d'1% toutes choses égales par ailleurs
proba_Hollande <- glm(I(Hollande>50) ~ Inscrits, data=res_2012, family = binomial(link = "probit")) # Régression probit
summary(proba_Hollande) # -9.7e-6* Résultat significatif mais dont la taille est non interprétable (cf. ci-dessous)
probitmfx(I(Hollande>50) ~ Inscrits, data=res_2012) # average marginal effect at mean: -3.8e-6*. Même résultat qu'au-dessus, mais interprétable:
# i.e. on average 38k (1.75sd) inscrits en plus augmentent la proba de victoire d'1% toutes choses égales par ailleurs (résultat semblable au modèle linéaire)

effet <- aad*mean(pop_circos)/(12660*ratio_pop_inscrits) # 1.14 Effet sur le score de Hollande (en %) associé à une population de circonscription réduite de l'écart moyen entre deux circonscriptions aléatoires
length(which(election2012$Hollande/(election2012$Hollande+election2012$Sarkozy) < 0.5+effet/100 & 0.5 < election2012$Hollande/(election2012$Hollande+election2012$Sarkozy)))/length(election2012$canton) 
# dans 4.6% des circos, Hollande a gagné avec moins de 51.14%, et il aurait donc perdu si la circo avait été plus grande


##### Résultats électoraux dans le redécoupage proposé #####



##### Sauvegarde #####
save.image("env.RData")

## TODO
# Comparer les résultats des dernières élections présidentielles avec le découpage actuel et mon découpage.
# Remplacer pop par majeurs
# Calculer les résultats électoraux sur Paris